import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;


public class Solution implements Iterable<UsedStock>{
	/** dobrota rješenja */
	private double fitness;
	/** zastavica koja označuje je li dobrota postavljena od zadnje promjene */
	private boolean fitnessUpdated;
	/** lista planova rezanja */
	private List<UsedStock> odabiri;
	
	/**
	 * Konstruktor koji inicijalizira listu planova rezanja. 
	 */
	public Solution() {
		odabiri = new LinkedList<UsedStock>();
		fitnessUpdated = false;
	}
	
	/**
	 * Vraća ukupnu količinu otpada za ovo rješenje.
	 * @return ukupna količina otpada
	 */
	public int getTotalWaste(){
		int waste = 0;
		for (UsedStock stock : odabiri) {
			waste += stock.getWaste();
		}
		return waste;
	}
	/**
	 * Metoda koja vraća dobrotu rješenja. Dobrota mora biti prethodno postavljena. 
	 * @return dobrota rješenja
	 * @throws IllegalStateException ako dobrota nije prethodno postavljena.
	 */
	public double getFitness() {
		if (!fitnessUpdated) {
			throw new IllegalStateException("Fitness nije ovježen od zadnje promjene.");
		}
		return fitness;
	}
	/**
	 * Metoda koja postavlja dobrotu rješenja. 
	 * @param fitness dobrota rješenja
	 */
	public void setFitness(double fitness) {
		this.fitness = fitness;
		fitnessUpdated = true;
	}
	/**
	 * Metoda koja vraća iterator po listi planova rezanja. Ne očekuje se da
	 * će se raditi ikakve promjene nad listom preko ovog iteratora. 
	 */
	public Iterator<UsedStock> iterator() {
		return odabiri.iterator();
	}
	/**
	 * Metoda koja dodaje plan rezanja u rješenje. 
	 * @param stock plan rezanja
	 * @return true ako je dodavanje uspjelo, false inače
	 */
	public boolean add(UsedStock stock) {
		return odabiri.add(stock);
	}
	/**
	 * Metoda koja vraća broj materijala korištenih u rješenju. 
	 * @return broj materijala u rješenju. 
	 */
	public int size() {
		return odabiri.size();
	}
}
