
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;


public class UsedStock {
	/** duljina stocka */
	private int stockSize;
	/** lista itema */
	private List<Integer> items;
	/** lista broja itema za svaki item*/
	private List<Integer> itemNumbers;
	/** ostatak bez gubitka pri rezanju */
	private int remainder;
	/** ostatak sa gubitkom pri rezanju */
	private int totalWaste;
	
	/**
	 * Konstruktor koji inicijalizira veličinu stocka, te ostatak i otpad. 
	 * @param stockSize veličina stocka
	 */
	public UsedStock(int stockSize) {
		this.stockSize = stockSize;
		this.remainder = stockSize;
		this.totalWaste = stockSize;
		
		items = new LinkedList<Integer>();
		itemNumbers = new ArrayList<Integer>();
	}
	
	/**
	 * Metoda koja vraća ukupnu duljinu stocka.
	 * @return duljina stocka
	 */
	public int getStockSize() {
		return this.stockSize;
	}
	/**
	 * Metoda koja vraća duljinu upotrebljivog ostatak stocka.
	 * @return upotrebljivi ostatak stocka
	 */
	public int getRemainder() {
		return this.remainder;
	}
	/**
	 * Metoda koja vraća količinu otpada stocka, tj. upotrebljivi ostatak i 
	 * gubitak pri rezanju. 
	 * @return količina otpada stocka
	 */
	public int getWaste() {
		return this.totalWaste;
	}
	/**
	 * Metoda koja pokušava dodati item duljine itemSize u plan rezanja stocka.
	 * @param itemSize duljina itema koji treba rezati
	 * @param cutLoss gubitak pri rezanju
	 * @return true ako je dodavanje uspješno, tj. ima dovoljno mjesta u stocku, 
	 * 		false inače
	 */
	public boolean addItem(int itemSize, int itemNumber, int cutLoss) {
		if (itemSize > remainder) return false;
		items.add(itemSize);
		itemNumbers.add(itemNumber);
		remainder -= itemSize;
		totalWaste -= itemSize;
		
		if(remainder != 0) {
			if (remainder > cutLoss) {
				remainder -= cutLoss;
			} else {
				remainder = 0;
			}
		}
		return true;
	}
	
	/**
	 * Metoda koja vraća item koji je na zadanom mjestu u listi.
	 * @param index broj itema koji treba vratiti
	 * @return item na mjestu itemNumber
	 */
	public int getItem(int index) {
		return items.get(index);
	}
	
	/**
	 * Metoda koja vraća broj itema na dobivenom indeksu. 
	 * @param index indeks za koji treba vratiti broj itema
	 * @return broj itema na indeksu index
	 */
	public int getItemNumber(int index) {
		return itemNumbers.get(index);
	}
	
	/**
	 * Metoda koja vraća broj itema koju su trenutno u planu rezanja stocka.
	 * @return
	 */
	public int numOfItems() {
		return items.size();
	}
	
	/**
	 * Metoda koja ispituje da li je stock neiskorišten. 
	 * @return true ako je neiskorišten, false inače
	 */
	public boolean isEmpty() {
		return stockSize == remainder;
	}
	
	/**
	 * Metoda koja vraća String koji opisuje ovaj plan rezanja. 
	 * @return string koji opisuje ovaj plan rezanja
	 */
	public String toString() {
		StringBuilder stringBuild = new StringBuilder(stockSize + "\t");
		for (int item : items) {
			stringBuild.append(item + "+");
		}
		return stringBuild.toString();
	}
}
