#!/usr/bin/perl -w

use strict;
use List::Util qw(shuffle sum);
use Data::Dumper;
use Clone qw(clone);
use Time::HiRes;

###################################################
#	Grouping Genetic Algorithm
#
# Program koristi grupirajući genetski algoritam za rješavanje
# jednodimenzionalnog problema rezanja sa više duljina materijala
# za rezanje (1/V/D/M tip prema Dyckhoffovoj topologiji)
#
# Autor: Ante Modrić
###################################################

my ($rez, $vel_pop, $vjer_kriz, $br_generacija, $max_loss, $elitizam, $vjer_mut, $max_vrijeme);

my (%stock, %items, @populacija);
my ($br_itema, $br_stock, $kol_itema);

my $problem_name = shift @ARGV;

{ package Kromosom;
	sub new{
		my $class = shift;
		my ($rjesenje, $cost) = @_;
		my $self = {rjesenje => $rjesenje, cost => $cost};
		bless $self, $class;
		return $self;
	}
	sub rjesenje		{$_[0]->{rjesenje}}
	sub cost		{$_[0]->{cost}}
	sub dobrota		{$_[0]->{dobrota}}
	sub set_dobrota		{$_[0]->{dobrota} = $_[1]}
}

sub konfiguracija{
	open KONF, "<algoritmi/groupingGA.conf" or die "ne mogu otvoriti konfiguracijsku datoteku\n";
	my $t = <KONF>;
	$t =~ /.+=(.+)/s;
	$vel_pop = $1;
	$t = <KONF>;
	$t =~ /.+=(.+)/s;
	$elitizam = $1;
	$t = <KONF>;
	$t =~ /.+=(.+)/s;
	$vjer_kriz = $1;
	$t = <KONF>;
	$t =~ /.+=(.+)/s;
	$vjer_mut = $1;
	$t = <KONF>;
	$t =~ /.+=(.+)/s;
	$br_generacija = $1;
	close KONF;
}

sub unos{
	open ULAZ, "<algoritmi/".$problem_name.".in" or die "ne mogu otvoriti ulaznu datoteku\n";
	my $ulaz;
	$ulaz = <ULAZ>;
	$ulaz =~ /(\d+)\t(\d+)\t(\d+)/;
	$max_vrijeme = $1;
	$rez = $2;
	$max_loss = $3;
	$br_stock = 0;
	$kol_itema = 0;
	$ulaz = <ULAZ>;
	chomp($ulaz);
	while($ulaz ne "----------"){
		$ulaz =~ /(\d+)\t(-?\d+)/;
		print STDERR "$1\t$2\n";
		if ($2 == -1){
			$stock{$br_stock} = [$1, 'inf'];
		} else {
			$stock{$br_stock} = [$1, $2];
		}
		$br_stock++;
		$ulaz = <ULAZ>;
		chomp($ulaz);
	}
	print STDERR "$ulaz\n";
	$ulaz = <ULAZ>;
	while($ulaz){
		chomp($ulaz);
		$ulaz =~ /(\d+)\t(\d+)/;
		print STDERR "$1\t$2\n";
		$items{$kol_itema} = [$1, $2];
		$kol_itema++;
		$br_itema += $2;
		$ulaz = <ULAZ>;
	}
	close ULAZ;
}

sub generiraj_populaciju{
	my @tmp;
	for(1..$vel_pop){	
		my $itemi = $br_itema;
		my %temp_stock = %{clone(\%stock)};
		my %temp_items = %{clone(\%items)};
		my @avail_stock = 0..($br_stock-1);
		my $temp_stock = $br_stock;
		my @ffd = sort {$items{$b}->[0] <=> $items{$a}->[0]} 0..($kol_itema-1);
		@ffd = shuffle 0..($kol_itema-1) if ($temp_stock == 1);
		my @rjesenje;
		my ($cost, $wasted, $br_novih);
		$wasted = 0;
		$br_novih = 0;
		while($itemi){
			my $a = int(rand($temp_stock));
			my $stock = $avail_stock[$a];
			if ($stock{$stock}->[1]=='inf'){
				$br_novih++;
			} else {
				$temp_stock{$stock}->[1]--;
				if ($temp_stock{$stock}->[1]==0){
					for($a..$#avail_stock){
						$avail_stock[$_] = $avail_stock[$_+1];
					}
					delete $avail_stock[-1];
					$temp_stock--;
				}
			}
			my $duljina = $stock{$stock}->[0];
			my @group;
			for(@ffd){
				next if ($items{$_}->[0] > $duljina);
				next if ($temp_items{$_}->[1]==0);
				push @group, $_;
				$duljina = $duljina - $items{$_}->[0] - $rez;
				$itemi--;
				$temp_items{$_}->[1]--;
				last if ($itemi==0);
				redo;
			}
			if ($duljina > $max_loss){
				$cost += ($duljina/$stock{$stock}->[0]);
				$wasted++;
			}
			next if (scalar(@group)==0);
			push @rjesenje, [\@group, $stock];
		}
		$cost = ($cost + ($wasted/scalar(@rjesenje)) + $br_novih)/scalar(@rjesenje);
		my $jedinka = Kromosom->new(\@rjesenje, $cost);
		push @tmp, $jedinka;
	}
	return @tmp;
}

sub selekcija{
	my $max = 0;
	for(@populacija){
		$max = $_->cost if ($_->cost > $max);
	}
	my $sd;
	for(@populacija){
		my $t = $max - $_->cost;
		$sd += $t;
		$_->set_dobrota($t);
	}
	my @duljine;
	for(0..$#populacija){
		$duljine[$_] = $populacija[$_]->dobrota / $sd;
	}
	
	my $r1;
	my $rand = rand(1);
	my $suma = 0;
	for(0..$#populacija){
		if ($suma+$duljine[$_] > $rand){
			$r1 = $_;
			last;
		}
		$suma += $duljine[$_];
	}
	my $r2 = $r1;
	until($r1 != $r2){
		$rand = rand(1);
		$suma = 0;
		for(0..$#populacija){
			if ($suma+$duljine[$_] > $rand){
				$r2 = $_;
				last;
			}
			$suma += $duljine[$_];
		}
	}
	return ($populacija[$r1],$populacija[$r2]);
}

sub krizaj{
	my ($r1, $r2) = @_;
	my $vel_segmenta = int (scalar(@{$r2->rjesenje}) * 0.4);
	my @segment;
	for(1..$vel_segmenta){
		my $t = int(rand(scalar(@{$r2->rjesenje})));
		my $postoji;
		for(@segment){
			$postoji = 1 if ($_ == $t);
		}
		redo if ($postoji);
		push @segment, $t;
	}
	my (@dijete, $br_novih, $cost, $wasted);
	$wasted = 0;
	$br_novih = 0;
	my $itemi = $br_itema;
	my %temp_stock = %{clone(\%stock)};
	my %temp_items = %{clone(\%items)};
	my @avail_stock = 0..($br_stock-1);
	my $temp_stock = $br_stock;
	my $index;
	for(@segment){
		my @t = @{$r2->rjesenje->[$_]};
		my @tmp = @{$t[0]};
		my $duljina = $stock{$t[1]}->[0];
		for(@tmp){
			$duljina = $duljina - $items{$_}->[0] - $rez;
			$temp_items{$_}->[1]--;
			$itemi--;
		}
		if ($duljina > $max_loss){
			$cost += ($duljina/$stock{$t[1]}->[0]);
			$wasted++;
		}
		if ($stock{$t[1]}->[1]=='inf'){
			$br_novih++;
		} else {
			$temp_stock{$t[1]}->[1]--;
			if ($temp_stock{$t[1]}->[1]==0){
				$index = 0;
				for(@avail_stock){
					last if ($t[1] == $_);
					$index++;
				}
				for($index..$#avail_stock){
					$avail_stock[$_] = $avail_stock[$_+1];
				}
				delete $avail_stock[-1];
				$temp_stock--;
			}
		}
		push @dijete, \@t;
	}
	my $x = 0;
	my $vel = scalar(@{$r1->rjesenje});
	for(@{$r1->rjesenje}){
		my @t = @{$_->[0]};
		my $t = $_->[1];
		next if ($temp_stock{$t}->[1]==0);
		my $next = 0;
		my %tmp;
		for(@t){
			$tmp{$_}++;
		}
		for(@t){
			$next = 1 if ($tmp{$_}>$temp_items{$_}->[1]);
		}
		if ($next){$x++;next;}
		my $duljina = $stock{$t}->[0];
		for(@t){
			$duljina = $duljina - $items{$_}->[0] - $rez;
			$temp_items{$_}->[1]--;
			$itemi--;
		}
		if ($duljina > $max_loss){
			$cost += ($duljina/$stock{$t}->[0]);
			$wasted++;
		}
		if ($stock{$t}->[1]=='inf'){
			$br_novih++;
		} else {
			$temp_stock{$t}->[1]--;
			if ($temp_stock{$t}->[1]==0){
				$index = 0;
				for(@avail_stock){
					last if ($t == $_);
					$index++;
				}
				for($index..$#avail_stock){
					$avail_stock[$_] = $avail_stock[$_+1];
				}
				delete $avail_stock[-1];
				$temp_stock--;
			}
		}
		push @dijete, $_;
	}
	my @ffd = sort {$items{$b}->[0] <=> $items{$a}->[0]} 0..($kol_itema-1);
	while($itemi){
		my $a = int(rand($temp_stock));
		my $stock = $avail_stock[$a];
		if ($stock{$stock}->[1]=='inf'){
			$br_novih++;
		} else {
			$temp_stock{$stock}->[1]--;
			if ($temp_stock{$stock}->[1]==0){
				for($a..$#avail_stock){
					$avail_stock[$_] = $avail_stock[$_+1];
				}
				delete $avail_stock[-1];
				$temp_stock--;
			}
		}
		my $duljina = $stock{$stock}->[0];
		my @group;
		for(@ffd){
			next if ($items{$_}->[0] > $duljina);
			next if ($temp_items{$_}->[1]==0);
			push @group, $_;
			$duljina = $duljina - $items{$_}->[0] - $rez;
			$itemi--;
			$temp_items{$_}->[1]--;
			last if ($itemi==0);
			redo;
		}
		if ($duljina > $max_loss){
			$cost += ($duljina/$stock{$stock}->[0]);
			$wasted++;
		}
		next if (scalar(@group)==0);
		push @dijete, [\@group, $stock];
	}
	$cost = ($cost + ($wasted/scalar(@dijete)) + $br_novih)/scalar(@dijete);
	my $jedinka = Kromosom->new(\@dijete, $cost);
	return $jedinka;
}

sub mutiraj{
	my $jedinka = shift;
	my @rjesenje = @{$jedinka->rjesenje};
	my $br_grupa = scalar(@rjesenje);
	my ($br_mutacija, $br_novih, $cost, $wasted);
	$wasted = 0;
	$br_novih = 0;
	for(1..$br_grupa){
		my $t = rand(1);
		$br_mutacija++ if ($t < $vjer_mut);
	}
	$br_mutacija++ if ($br_mutacija==1);
	my @mutacije;
	for(1..$br_mutacija){
		my $a = int(rand($br_grupa));
		my $postoji;
		for(@mutacije){
			$postoji = 1 if ($_ == $a);
		}
		redo if ($postoji);
		push @mutacije, $a;
	}
	my @dijete;
	my $itemi = $br_itema;
	my %temp_stock = %{clone(\%stock)};
	my %temp_items = %{clone(\%items)};
	for(0..$#rjesenje){
		my $t = $_;
		my $postoji;
		for(@mutacije){
			$postoji = 1 if ($_ == $t);
		}
		next if ($postoji);
		my @t = @{$rjesenje[$_]->[0]};
		$t = $rjesenje[$_]->[1];
		my $duljina = $stock{$t}->[0];
		for(@t){
			$duljina = $duljina - $items{$_}->[0] - $rez;
			$temp_items{$_}->[1]--;
			$itemi--;
		}
		if ($duljina > $max_loss){
			$cost += ($duljina/$stock{$t}->[0]);
			$wasted++;
		}
		if ($stock{$t}->[1]=='inf'){
			$br_novih++;
		} else {
			$temp_stock{$t}->[1]--;
		}
		push @dijete, $rjesenje[$_];
	}
	my @ffd = sort {$items{$b}->[0] <=> $items{$a}->[0]} 0..($kol_itema-1);
	while($itemi){
		my $stock = int(rand($br_stock));
		next if ($temp_stock{$stock}->[1]==0);
		if ($stock{$stock}->[1]=='inf'){
			$br_novih++;
		} else {
			$temp_stock{$stock}->[1]--;
		}
		my $duljina = $stock{$stock}->[0];
		my @group;
		for(@ffd){
			next if ($items{$_}->[0] > $duljina);
			next if ($temp_items{$_}->[1]==0);
			push @group, $_;
			$duljina = $duljina - $items{$_}->[0] - $rez;
			$itemi--;
			$temp_items{$_}->[1]--;
			last if ($itemi==0);
			redo;
		}
		if ($duljina > $max_loss){
			$cost += ($duljina/$stock{$stock}->[0]);
			$wasted++;
		}
		next if (scalar(@group)==0);
		push @dijete, [\@group, $stock];
	}
	$cost = ($cost + ($wasted/scalar(@dijete)) + $br_novih)/scalar(@dijete);
	my $rezultat = Kromosom->new(\@dijete, $cost);
	return $rezultat;
}

sub nova_generacija{
	my $br_starih = int($vel_pop*$elitizam);
	my @tmp = sort { $a->cost <=> $b->cost } @populacija;
	my @nova_generacija;
	for(1..$br_starih){
		push @nova_generacija, (shift @tmp);
	}
	my $br_novih = $vel_pop - $br_starih;
	for(1..($br_novih/2)){
		my ($r1, $r2) = &selekcija();
		my $a = rand(1);
		my ($d1, $d2);
		if ($a < $vjer_kriz){
			$d1 = &krizaj($r1, $r2);
			$d2 = &krizaj($r1, $r2);
		} else {
			$d1 = &mutiraj($r1);
			$d2 = &mutiraj($r2);
		}
		push @nova_generacija, $d1;
		push @nova_generacija, $d2;
	}
	return @nova_generacija;
}

sub ispisi_rezultat{
	my $rezultat = shift;
	open IZLAZ, ">>rjesenja/groupingGA_$problem_name.out" or die "nisam nasao izlaznu datoteku\n";
	select IZLAZ;
	print "##########\n";
	my ($zbr_stock, $zbr_itemi, $br_novih);
	for(@{$rezultat->rjesenje}){
		my $stock = $_->[1];
		my @group = @{$_->[0]};
		my $duljina = $stock{$stock}->[0];
		$zbr_stock += $duljina;
		$br_novih++ if ($stock{$stock}->[1]=='inf');
		print "$duljina\t";
		for(@group){
			$duljina = $duljina - $items{$_}->[0] - $rez;
			$zbr_itemi += $items{$_}->[0];
			print $items{$_}->[0]."+";
		}
		print "\n";
	}
	
	close IZLAZ;
}

##################
# Glavni program #
##################

&konfiguracija();
&unos();
@populacija = &generiraj_populaciju();
my $vrijeme = Time::HiRes::time();
for(1..$br_generacija){
	last if ((Time::HiRes::time() - $vrijeme) >= $max_vrijeme);
	@populacija = &nova_generacija();
	my $max = 0;
	for(@populacija){
		$max = $_->cost if ($_->cost > $max);
	}
	my $sd;
	for(@populacija){
		my $t = $max - $_->cost;
		$sd += $t;
	}
	last if ($sd == 0);
}
my @tmp = sort { $a->cost <=> $b->cost } @populacija;
&ispisi_rezultat(shift @tmp);
